/* 
	Win32 I2C master-transmitter/master-receiver routines 
	asgaut@pvv.org/97-06-24

	Requires this RS-232 <-> I2C interface behavior:
	DTR -> SCL (DTR active => SCL pulled low)
	DSR <- SCL (SCL low => DSR active, SCL high => DSR inactive)
	RTS -> SDA (RTS active => SDA pulled low)
	CTS <- SDA (SDA low => CTS active, SDA high => CTS inactive)

	I2C bit rate on a PPro 180: 580 Hz
*/


#include "i2c.h"


#define SCL_LOW		DTR_CONTROL_ENABLE
#define SCL_HIGH	DTR_CONTROL_DISABLE
#define SDA_LOW		RTS_CONTROL_ENABLE
#define SDA_HIGH	RTS_CONTROL_DISABLE


static DWORD I2CSCL(I2CContext *pcI2C, DWORD dwNewState)
{
	DWORD dwModemStatus;

	pcI2C->dcb.fDtrControl = dwNewState; 
	SetCommState(pcI2C->hSerialPort, &pcI2C->dcb);
	GetCommModemStatus(pcI2C->hSerialPort, &dwModemStatus);
	return (dwModemStatus&MS_DSR_ON)?SCL_LOW:SCL_HIGH;
}

static DWORD I2CSDA(I2CContext *pcI2C, DWORD dwNewState)
{
	DWORD dwModemStatus;

	pcI2C->dcb.fRtsControl = dwNewState; 
	SetCommState(pcI2C->hSerialPort, &pcI2C->dcb);
	GetCommModemStatus(pcI2C->hSerialPort, &dwModemStatus);
	return (dwModemStatus&MS_CTS_ON)?SDA_LOW:SDA_HIGH;
}

static BOOL I2CStart(I2CContext *pcI2C)
{
	I2CSDA(pcI2C, SDA_LOW);
	I2CSCL(pcI2C, SCL_LOW);
	return TRUE;
}

static BOOL I2CStop(I2CContext *pcI2C)
{
	I2CSDA(pcI2C, SDA_LOW);
	I2CSCL(pcI2C, SCL_HIGH);
	I2CSDA(pcI2C, SDA_HIGH);
	return TRUE;
}

// Return value: TRUE if data acknowledged on I2C bus. FALSE if not acknowledged.

static BOOL I2CPutByte(I2CContext *pcI2C, unsigned char byte)
{
	DWORD dwSDA;
	int i;

	I2CSCL(pcI2C, SCL_LOW);
	for (i=7; i>=0; i--)
	{
		I2CSDA(pcI2C, (byte & (1<<i))?SDA_HIGH:SDA_LOW);
		I2CSCL(pcI2C, SCL_HIGH);
		I2CSCL(pcI2C, SCL_LOW);
	}
	I2CSDA(pcI2C, SDA_HIGH);
	I2CSCL(pcI2C, SCL_HIGH);
	dwSDA = I2CSDA(pcI2C, SDA_HIGH);
	I2CSCL(pcI2C, SCL_LOW);

	return dwSDA==SDA_LOW?TRUE:FALSE;
}

static BOOL I2CGetByte(I2CContext *pcI2C, unsigned char *byte, BOOL bSendAck)
{
	DWORD dwSDA;
	int i;

	I2CSDA(pcI2C, SDA_HIGH); // release SDA
	*byte = 0;
	for (i=7; i>=0; i--)
	{
		I2CSCL(pcI2C, SCL_HIGH);
		dwSDA = I2CSDA(pcI2C, SDA_HIGH);
		if (dwSDA==SDA_HIGH)
			*byte |= 1<<i;
		I2CSCL(pcI2C, SCL_LOW);
	}
	if (bSendAck)
		I2CSDA(pcI2C, SDA_LOW);
	I2CSCL(pcI2C, SCL_HIGH);
	I2CSCL(pcI2C, SCL_LOW);
	return TRUE;
}

// Public functions

BOOL I2COpen(LPCTSTR portname, I2CContext *pcI2C)
{
	HANDLE hPort;

	memset(pcI2C, 0, sizeof(I2CContext));
	hPort = CreateFile(portname, GENERIC_READ|GENERIC_WRITE,0,0,OPEN_EXISTING,FILE_FLAG_OVERLAPPED,0);
	if (hPort != INVALID_HANDLE_VALUE) {
		pcI2C->hSerialPort = hPort;
		if (!GetCommState(pcI2C->hSerialPort, &pcI2C->dcb)) 
			return FALSE;
	}
	else return FALSE;
	return TRUE;
}

BOOL I2CClose(I2CContext *pcI2C)
{
	return CloseHandle(pcI2C->hSerialPort);
}

BOOL I2CTransmitData(I2CContext *pcI2C, BOOL bSendStop, unsigned char addr, int nBytes, void *pSrc)
{
	int i;
	BOOL bAck;

	I2CStart(pcI2C);
	I2CPutByte(pcI2C, (unsigned char)(addr & 0xfe));
	for (i=0; i<nBytes; i++)
		bAck = I2CPutByte(pcI2C, *(((unsigned char*)pSrc)+i));
	if (bSendStop)
		I2CStop(pcI2C);
	else 
	{
		I2CSDA(pcI2C, SDA_HIGH);
		I2CSCL(pcI2C, SCL_HIGH);
	}
	return bAck;
}

BOOL I2CReceiveData(I2CContext *pcI2C, unsigned char addr, int nBytes, void *pDest)
{
	int i;

	I2CStart(pcI2C);
	I2CPutByte(pcI2C, (unsigned char)(addr | 0x01));
	for (i=0; i<nBytes; i++)
		I2CGetByte(pcI2C, ((unsigned char*)pDest)+i, i<nBytes-1);
	I2CStop(pcI2C);
	return TRUE;
}

